--
-- Fuel Useability for motorized vehicles Specialization
-- by Blacky_BPG
-- For FS15 only
--
-- Version 5.15.7 | 30.11.2014	- fixed HUD placement for drivingControl (Thanks IntruderVS1400 and Klaumann2001)
							-- add calculation for turned on implements and filled equipments
-- Version 5.15.6 | 29.11.2014	- fixed HUD placement and font size for different resolutions
-- Version 5.15.5 | 28.11.2014	- fixed compatibility with drivingControl
-- Version 5.15.4 | 08.11.2014
--

fuelAdjust = {};
fuelAdjust.modDir = g_currentModDirectory;
local misha = 0;
local misdiff = 0;
local lastha = 0;
local lastfac = 0;
local lastdiff = 0;
local lasttime = 0;

function fuelAdjust.prerequisitesPresent(specializations)
	return true;
end;
function fuelAdjust:load(xmlFile)
	self.fuelAdjust = {};
	self.fa = {};
	self.fa.backupmaxTorque = 0.5;
	self.fa.backupminRpm = 1000;
	self.fa.backupmaxRpm = 3000;
	self.fa.fuelUsage = self.fuelUsage * 30;
	self.fuelUsage = 0;  -- Override for recalculation
	self.needFuel = Utils.getNoNil(getXMLBool(xmlFile, "vehicle.needFuel"), true);
	if self.motor ~= nil then
		if self.motor.maxTorque ~= nil then
			self.fa.backupmaxTorque = self.motor.maxTorque;
		end;
		if self.motor.minRpm ~= nil then
			self.fa.backupminRpm = self.motor.minRpm;
		end;
		if self.motor.maxRpm ~= nil then
			self.fa.backupmaxRpm = self.motor.maxRpm;
		end;
	end;
	self.fa.lastFuelFill = self.fuelFillLevel;

	self.addfuelAdjust = SpecializationUtil.callSpecializationsFunction("addfuelAdjust");

	self.fa.fuelUsed = 0;
	self.fa.fuelUsageSum = 0;
	self.fa.fuelUsageSumTmp = 0;
	self.fa.counter = 0;
	self.fa.OverallFuelUsage = 0;
	self.fa.OverallFuel = 0;
	self.fa.OverallFuelUsageFactor = 0;
	self.fa.fixStartFuel = -1;
	self.fa.fuelUsage = self.fa.fuelUsage / self.fa.backupmaxTorque
	self.fa.fuelUsageBackup = self.fa.fuelUsage;
	self.fa.idleUsage = self.fa.fuelUsage * 0.05;
	self.fa.maxFuelUse = self.fa.idleUsage + ((self.fa.backupmaxTorque * self.fa.fuelUsage) * 1.5);
	self.fa.colorMode = 0;
end;

function fuelAdjust:update(dt)
	if self.isServer then
	if self.isMotorStarted then
		if self.fuelFillLevel ~= nil and self.fuelFillLevel <= 0.1 then
			if self.isHired then
				self:dismiss();
			end;
			if self.driveControl ~= nil and self.driveControl.manMotorStart ~= nil and self.driveControl.manMotorStart.isActive then
				if self.driveControl.manMotorStart.isMotorStarted then
					driveControl:motorStop(self,false);
				end;
			end;
		end;
	else
		if self.fa ~= nil and self.fa.fixStartFuel ~= nil and self.fa.fixStartFuel > -1 then
			if self.fuelFillLevel ~= nil and self.fuelFillLevel > self.fa.fixStartFuel then
				local diff = self.fuelFillLevel - self.fa.fixStartFuel;
				local delta = diff * g_fuelPricePerLiter;
				g_currentMission.missionStats.statistics.expenses.total = g_currentMission.missionStats.statistics.expenses.total - delta;
				g_currentMission.missionStats.statistics.expenses.session = g_currentMission.missionStats.statistics.expenses.session - delta;
				g_currentMission:addSharedMoney(delta, "vehicleRunningCost");
				self:setFuelFillLevel(self.fa.fixStartFuel);
				self.fa.fixStartFuel = -1;
			end;
		end;
	end;
	if self.isFuelFilling then
		self.fa.lastFuelFill = self.fuelFillLevel;
	end;
	if self.needFuel and (self:getIsActive() or self.isHired) then
	  if self.isMotorStarted then
		self.fa.fuelUsed = 0;
		self.fuelUsage = 0;  -- Override for recalculation

		self.fa.fuelUsed = self.fa.idleUsage + ((self.motor.motorLoad * self.fa.fuelUsage) * (self.motor.lastMotorRpm / self.fa.backupmaxRpm * 2));

		if self.movingDirection ~= 0 then
			local fLevel, capacity = self:getAttachedTrailersFillLevelAndCapacity();
			if fLevel ~= nil then
				if fLevel > 0 then
					self.fa.fuelUsed = self.fa.fuelUsed + (fLevel * 0.0000000017);
				end;
			end;
			if self.driveControl ~= nil and g_currentMission.driveControl.useModules.fourWDandDifferentials and not self.driveControl.fourWDandDifferentials.isSurpressed then
				if self.driveControl.fourWDandDifferentials.fourWheel then
					self.fa.fuelUsed = self.fa.fuelUsed + (self.fa.fuelUsage * 0.02125);
				end;
				if self.driveControl.fourWDandDifferentials.diffLockFront then
					self.fa.fuelUsed = self.fa.fuelUsed + (self.fa.fuelUsage * 0.00708);
				end;
				if self.driveControl.fourWDandDifferentials.diffLockBack then
					self.fa.fuelUsed = self.fa.fuelUsed + (self.fa.fuelUsage * 0.00708);
				end;
			end;
		end;

		for i=1, table.getn(self.attachedImplements) do
			local implement = self.attachedImplements[i].object;
			if implement.isTurnedOn == true then
				self.fa.fuelUsed = self.fa.fuelUsed + (self.fa.fuelUsage * 0.0075);
			end;
			if self.movingDirection ~= 0 then
				for x = 1, table.maxn(implement.specializations) do
					local vs = implement.specializations[x];
					if vs ~= nil and vs == SpecializationUtil.getSpecialization("cultivator") then
						if implement.cultivatorHasGroundContact then
							self.fa.fuelUsed = self.fa.fuelUsed + (self.fa.fuelUsage * 0.015);
						end;
					end;
					if vs ~= nil and vs == SpecializationUtil.getSpecialization("plough") then
						if implement.ploughHasGroundContact then
							self.fa.fuelUsed = self.fa.fuelUsed + (self.fa.fuelUsage * 0.0217);
						end;
					end;
					if vs ~= nil and vs == SpecializationUtil.getSpecialization("sowingMachine") then
						if implement.sowingMachineHasGroundContact then
							self.fa.fuelUsed = self.fa.fuelUsed + (self.fa.fuelUsage * 0.0117);
						end;
					end;
				end;
			end;
		end;

		if self.isThreshing or self.isAIThreshing then
			self.fa.fuelUsed = self.fa.fuelUsed + (self.fa.fuelUsage * 0.0075);
		end;

		if self.fuelFillLevel > 0 then
			if self.fa.counter >= 30 then
				self.fa.fuelUsageSum = (self.fa.fuelUsageSumTmp + (self.fa.fuelUsed * 3.3334)) / (self.fa.counter + 1);
				self.fa.fuelUsageSumTmp = self.fa.fuelUsageSum;
				self.fa.OverallFuelUsageFactor = self.fa.OverallFuelUsageFactor + 1;
				self.fa.OverallFuelUsage = ((self.fa.OverallFuelUsage * (self.fa.OverallFuelUsageFactor - 1)) + (self.fa.fuelUsageSum * 10000)) / self.fa.OverallFuelUsageFactor;
				self.fa.counter = 0;
				self:addfuelAdjust(self.fa.OverallFuelUsage, self.fa.OverallFuelUsageFactor, self.fa.OverallFuel, self.fa.fuelUsageSum);
			else
				self.fa.fuelUsageSumTmp = self.fa.fuelUsageSumTmp + (self.fa.fuelUsed * 3.3334);
				self.fa.counter = self.fa.counter + 1;
			end;
			self.fa.lastFuelFill = self.fa.lastFuelFill - self.fa.fuelUsed;
			if self.isServer then
				self:setFuelFillLevel(self.fa.lastFuelFill);
				g_currentMission.missionStats.statistics.fuelUsage.total = g_currentMission.missionStats.statistics.fuelUsage.total + self.fa.fuelUsed;
				g_currentMission.missionStats.statistics.fuelUsage.session = g_currentMission.missionStats.statistics.fuelUsage.session + self.fa.fuelUsed;
			end;
			self.fa.OverallFuel = self.fa.OverallFuel + (self.fa.fuelUsed * 1.6667);
		else
			if self.stopAITractor ~= nil then
				self:stopAITractor();
			elseif self.stopAIThreshing ~= nil then
				self:stopAIThreshing();
			end;
		end;
	  else
		if self.fa then
			if self.fa.fuelUsageSum > 0 then
				self.fa.fuelUsageSum = (self.fa.fuelUsageSumTmp + (self.fa.fuelUsed * 3.3334)) / (self.fa.counter + 1);
				self.fa.fuelUsageSumTmp = self.fa.fuelUsageSum;
				self.fa.OverallFuelUsageFactor = self.fa.OverallFuelUsageFactor + 1;
				self.fa.OverallFuelUsage = ((self.fa.OverallFuelUsage * (self.fa.OverallFuelUsageFactor - 1)) + (self.fa.fuelUsageSum * 10000)) / self.fa.OverallFuelUsageFactor;
				self.fa.counter = 0;
				self.fa.lastFuelFill = self.fa.lastFuelFill - self.fa.fuelUsed;
				if self.isServer then
					self:setFuelFillLevel(self.fa.lastFuelFill);
					g_currentMission.missionStats.statistics.fuelUsage.total = g_currentMission.missionStats.statistics.fuelUsage.total + self.fa.fuelUsed;
					g_currentMission.missionStats.statistics.fuelUsage.session = g_currentMission.missionStats.statistics.fuelUsage.session + self.fa.fuelUsed;
				end;
				self.fa.OverallFuel = self.fa.OverallFuel + (self.fa.fuelUsed * 3.3334);
				self:addfuelAdjust(self.fa.OverallFuelUsage, self.fa.OverallFuelUsageFactor, self.fa.OverallFuel, self.fa.fuelUsageSum);
			end;
			self.fa.fuelUsageSum = 0;
		end;
	  end;
	end;
	end;
end;
function fuelAdjust:updateTick(dt)
	if lasttime == 0 then
		lasttime = math.random(180,220);
		if self.isMotorStarted then
			self:addfuelAdjust(self.fa.OverallFuelUsage, self.fa.OverallFuelUsageFactor, self.fa.OverallFuel, self.fa.fuelUsageSum);
			if self.fa.OverallFuelUsageFactor > 20000 then
				self.fa.OverallFuelUsage = ((self.fa.OverallFuelUsage / self.fa.OverallFuelUsageFactor) * 5000);
				self.fa.OverallFuelUsageFactor = 5000;
			end;
		end;
	else
		lasttime = lasttime - 1;
	end;
end;
function fuelAdjust:readStream(streamId, connection)
	self.fa.OverallFuelUsage = streamReadFloat32(streamId);
	self.fa.OverallFuelUsageFactor = streamReadInt32(streamId);
	self.fa.OverallFuel = streamReadFloat32(streamId);
	self.fa.fuelUsageSum = streamReadFloat32(streamId);
	self.fa.colorMode = streamReadInt8(streamId);
end;

function fuelAdjust:writeStream(streamId, connection)
	streamWriteFloat32(streamId, self.fa.OverallFuelUsage);
	streamWriteInt32(streamId, self.fa.OverallFuelUsageFactor);
	streamWriteFloat32(streamId, self.fa.OverallFuel);
	streamWriteFloat32(streamId, self.fa.fuelUsageSum);
	streamWriteInt8(streamId, self.fa.colorMode);
end;
function fuelAdjust:addfuelAdjust(addOFU,addOFUF, addFU, fuS, cM, noEventSend)
	addOFU = math.max(Utils.getNoNil(addOFU, 0.0), 0.0);
	addOFUF = math.max(Utils.getNoNil(addOFUF, 0.0), 0.0);
	addFU = math.max(Utils.getNoNil(addFU, 0.0), 0.0);
	fuS = math.max(Utils.getNoNil(fuS, 0.0), 0.0);

	if self.isServer then
		local pA = self.fa.maxFuelUse * 0.75;
		local pB = self.fa.maxFuelUse * 0.95;
		if self.fa.fuelUsed > pB then
			self.fa.colorMode = 2;
		elseif self.fa.fuelUsed > pA then
			self.fa.colorMode = 1;
		else
			self.fa.colorMode = 0;
		end;
		cM = self.fa.colorMode;
	else
		self.fa.colorMode = cM;
	end;

	fuelAdjustEvent.sendEvent(self, addOFU, addOFUF, addFU, fuS, cM, noEventSend);

	self.fa.OverallFuelUsage = addOFU;
	self.fa.OverallFuelUsageFactor = addOFUF;
	self.fa.OverallFuel = addFU;
	self.fa.fuelUsageSum = fuS;
end;
function fuelAdjust:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	local addfusage = Utils.getNoNil(getXMLFloat(xmlFile, key .. "#OverallFuelUsage"), self.fa.OverallFuelUsage);
	local addfusagefactor = Utils.getNoNil(getXMLFloat(xmlFile, key .. "#OverallFuelUsageFactor"), self.fa.OverallFuelUsageFactor);
	local addfused = Utils.getNoNil(getXMLFloat(xmlFile, key .. "#OverallFuelUsed"), self.fa.OverallFuel);
	local ffl = Utils.getNoNil(getXMLFloat(xmlFile, key .. "#fuelFillLevel"), self.fa.lastFuelFill);
	self.fa.OverallFuelUsage = addfusage;
	self.fa.OverallFuelUsageFactor = addfusagefactor;
	self.fa.OverallFuel = addfused;
	self.fa.lastFuelFill = ffl;
	self:addfuelAdjust(self.fa.OverallFuelUsage, self.fa.OverallFuelUsageFactor, self.fa.OverallFuel);
	local fuelFillLevel = getXMLFloat(xmlFile, key.."#fuelFillLevel");
	self.fa.fixStartFuel = fuelFillLevel;
	return BaseMission.VEHICLE_LOAD_OK;
end;

function fuelAdjust:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'OverallFuelUsage="' .. tostring(self.fa.OverallFuelUsage) ..'" OverallFuelUsageFactor="' .. tostring(self.fa.OverallFuelUsageFactor) ..'" OverallFuelUsed="' ..tostring(self.fa.OverallFuel) ..'"';
	return attributes, nil;
end;

function fuelAdjust:onEnter()
end;

function fuelAdjust:onLeave()
end;

function fuelAdjust:draw()
	if self:getIsActive() then
		if SpecializationUtil.hasSpecialization(Motorized, self.specializations) then
			if self.needFuel then
				-- setTextBold(true);
				setTextAlignment(RenderText.ALIGN_RIGHT);
				if self.fa.colorMode == 2 then
					setTextColor(1,0,0,1);
				elseif self.fa.colorMode == 1 then
					setTextColor(1,1,0,1);
				else
					setTextColor(1,1,1,1);
				end;
				renderText(0.9580, g_currentMission.fuelLevelStatusBarPosY - 0.0605, g_currentMission.fillLevelTextSizeSmall, string.format("%.2f", self.fa.fuelUsageSum * 10000 * 1.166666666) .. " ltr");

				-- setTextBold(true);
				setTextAlignment(RenderText.ALIGN_RIGHT);
				setTextColor(0.25,0.25,0.25,1);
				renderText(0.9580, g_currentMission.fuelLevelStatusBarPosY - 0.0705, g_currentMission.fillLevelTextSizeSmall, string.format("%.2f", self.fa.OverallFuelUsage) .. " ltr");
			end;
		end;
	end;
end;

function fuelAdjust:delete()
end;

function fuelAdjust:mouseEvent(posX, posY, isDown, isUp, button)
end;

function fuelAdjust:keyEvent(unicode, sym, modifier, isDown)
end;

Motorized.update = Utils.appendedFunction(fuelAdjust.update, Motorized.update);

-----------------------------------------------------------------------

fuelAdjustEvent = {};
fuelAdjustEvent_mt = Class(fuelAdjustEvent, Event);

InitEventClass(fuelAdjustEvent, "fuelAdjustEvent");

function fuelAdjustEvent:emptyNew()
    local self = Event:new(fuelAdjustEvent_mt);
    self.className="fuelAdjustEvent";
    return self;
end;

function fuelAdjustEvent:new(vehicle, addOFU, addOFUF, addFU, fuS, cM)
	local self = fuelAdjustEvent:emptyNew()
	self.vehicle = vehicle;
	self.OverallFuelUsage = addOFU;
	self.OverallFuelUsageFactor = addOFUF;
	self.OverallFuel = addFU;
	self.FuelUsageSum = fuS;
	self.colorMode = cM;
	return self;
end;

function fuelAdjustEvent:readStream(streamId, connection)
	local id = streamReadInt32(streamId);
	self.OverallFuelUsage = streamReadFloat32(streamId);
	self.OverallFuelUsageFactor = streamReadInt32(streamId);
	self.OverallFuel = streamReadFloat32(streamId);
	self.FuelUsageSum = streamReadFloat32(streamId);
	self.colorMode = streamReadInt8(streamId);
	self.vehicle = networkGetObject(id);
	self:run(connection);
end;

function fuelAdjustEvent:writeStream(streamId, connection)
	streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
	streamWriteFloat32(streamId, self.OverallFuelUsage);
	streamWriteInt32(streamId, self.OverallFuelUsageFactor);
	streamWriteFloat32(streamId, self.OverallFuel);
	streamWriteFloat32(streamId, self.FuelUsageSum);
	streamWriteInt8(streamId, self.colorMode);
end;

function fuelAdjustEvent:run(connection)
	if self.vehicle ~= nil then
		if not connection:getIsServer() then
			g_server:broadcastEvent(fuelAdjustEvent:new(self.vehicle, self.OverallFuelUsage, self.OverallFuelUsageFactor, self.OverallFuel, self.FuelUsageSum, self.colorMode), nil, connection, self.vehicle);
		end;
		self.vehicle:addfuelAdjust(self.OverallFuelUsage, self.OverallFuelUsageFactor, self.OverallFuel, self.FuelUsageSum, self.colorMode, true);
	end;
end;

function fuelAdjustEvent.sendEvent(vehicle, OFU, OFUF, FU, FUS, CM, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(fuelAdjustEvent:new(vehicle, OFU, OFUF, FU, FUS, CM), nil, nil, vehicle);
		end;
	end;
end;
